
## Improving Topic Modeling through Homophily for Legal Documents

Python code for [Improving Topic Modeling through Homophily for Legal Documents]()

### Setup

1. Get the RTM model from [here](https://github.com/dongwookim-ml/python-topic-model).

2. Install dependencies:
    - Python 2.7
    - nltk (`pip install nltk`)
    - gensim (`pip install gensim`)
    - numpy  (`pip install numpy`)
    - matplotlib (`pip install matplotlib`)

### Prepare the data

1. Build the corpus from text documents
    - `python make_corpus_data.py`
    - `python make_link_data.py`
    - `python make_link.py`

2. Build Homophily graph using `exchange_cocite.py` a scipt to choose prior cases and status above a weight threshold.
    - `value_limit_lis` is for setting the case and status weight thresholds. Those with weights above the thresholds will be used.
        (For example: setting as [0, 0] will use all the cases and status.)
    - Set `data_name` to `both` (use both case and status), `case` (use `case` only) or `status` (use `status` only)
    - Set `data_mode` to `origin` for using the `wn` method, `change` for using the `wj` method and `wn2v` for `ws`as described in the paper. 
    - Execute this script with the set parameters, it will create the case and status data for training the RTM model.

3. Train Node2vec
    - Clone the node2vec model from [here](https://github.com/aditya-grover/node2vec.git) in the same directory (if not available).
    - Run the script bash for node2vec : `bash make_node2vec.sh`, the results will be putted in models.
    - All the hyper-parameters for node2vec can be changed in `make_node2vec.sh`

### Train RTM/LDA

- For RTM run: `python make_rtm.py`
- This script use the created case and status data for training the RTM model and output topics using the following hyper-parameters:
    - `corpus` is for setting the directory for parameter output
    - `path_data` is for setting the directory for topic output
    - `max_iter` maximum iteration is set as 10 by default
    - `journal_mode`
        - for the experiments of the ANS journal submission
        - 0: for the cutting model 
        - 1: case only
        - 2: status only
        - 3: case + status
        - 4: case * status
        - 5: case * status/(status + case * status )=> Hamacher T-norm
        - 6: max(0,status + case-1) => Yeger T-norm
        - 7: max(status, case) 
        - 8: min(status, case)
        - 9: status * case/(1+(1-status)*(1-case)) => Hamacher T-norm
- For LDA run: `python make_lda.py`
- Outputs:
    - Parameters will be output to `./{corpus}/`
    - Topic will be output to `./output/{path_data}`
    
### Test RTM/LDA

- Evaluate the coherence of the output topics
- Put a pretrained word embedding model to `./eval_model`
    - As it is large, plese download it by yourself
    - GloVe can be downloaded from `http://nlp.stanford.edu/data/glove.840B.300d.zip`
    - It is also possible to use other pretrained word embedding models
- Set `embedding_name` to the pretrained word embedding model
- Set `text_name` to the output topics
- Execute the script :  `python evaluation.py`
    - It will output the coherence
- To evaluate the model with Cv, UMass and UCI :
    - `python allcoherence.py [path_to_topics]`