
import numpy as np
import sys

class LoadModel():
    def __init__(self, model_name_ori):
        # model_name = '{}.model'.format(model_name_ori)
        # model_path = '../model/{}'.format(model_name)
        # print(model_path)

        model_path =model_name_ori# './eval_model/{}.model'.format(model_name)
        print(model_path)

        with open(model_path, 'r') as f:
            if 'glove' in model_name_ori  :
                n_vocab, n_units = 2196017, 300
            else:
                ss = f.readline().split()
                n_vocab, n_units = int(ss[0]), int(ss[1])
            word2index = {}
            index2word = {}
            vocabulary = set()
            w = np.empty((n_vocab, n_units), dtype=np.float32)
            for i, line in enumerate(f):
                ss = line.split()
                if len(ss) != 301:
                    continue
                word = ss[0]
                vocabulary.add(word)
                word2index[word] = i
                index2word[i] = word
                w[i] = np.array([float(s) for s in ss[1:]], dtype=np.float32)
        s = np.sqrt((w * w).sum(1))
        w /= s.reshape((s.shape[0], 1))  # normalize

        self.vector = w
        self.index2word = index2word
        self.word2index = word2index
        self.vocabulary = vocabulary

def cos_sim(vec_a, vec_b):
    norm_ab = np.linalg.norm(vec_a) * np.linalg.norm(vec_b)
    if norm_ab != 0:
        return np.dot(vec_a, vec_b) / norm_ab
    else:
        # ベクトルのノルムが0だと似ているかどうかの判断すらできないので最低値
        return -1

def get_vec(model, word):
    index = model.word2index[word]
    vec = model.vector[index]
    return vec


def calc_cohe(text_name, model):
    print(text_name)
    coherence = 0

    with open(text_name, 'r') as f:
        count = 0
        for line in f:
            line = line.strip().split('\t')
            now_coh = 0
            cal_data_line = line[:11]
            count_calc = 0
            for i, word1 in enumerate(cal_data_line[1:]):
                for word2 in cal_data_line[i + 2:]:
                    if word1 in model.vocabulary and word2 in model.vocabulary:
                        vec1 = get_vec(model, word1)
                        vec2 = get_vec(model, word2)
                        now_coh += cos_sim(vec1, vec2)
                        count_calc += 1
            # now_coh /= (len(line) - 1) * (len(line) - 2) / 2
            if count_calc > 0:
                now_coh /= count_calc
                coherence += now_coh
                count += 1
    # print(round(coherence / count, 4))
    return round(coherence / count, 4)


def chekck(mode, corpus, model, corpus2):
    if mode == 'rtm':
        text_name = '../data/{}_{}{}.txt'.format(mode, corpus, corpus2)
    else:
        if corpus2 != '':
            return
        text_name = '../data/{}_{}.txt'.format(mode, corpus)
    text_name = '../data/{}.txt'.format(corpus2)
    calc_cohe(text_name)



def main():
    embedding_name = '/mnt/nfs/nas4/cicoda_tmp/cicoda_tmp/Osaka/data/glove.840B.300d.txt'
    text_name = './output/output1/10all.txt'

    model = LoadModel(embedding_name)
    calc_cohe(text_name, model)

    # sys.exit()
    #
    #
    # model_list = ['glove', 'w2v']
    # mode_list = ['lda', 'rtm']
    # corpus_list = ['all', 'summary']
    # # corpus_list2 = ['', '_co_st_0', '_co_st_5', '_co_st_10', '_co_10000']
    # # corpus_list2 = ['_co_case_change_0.25', '_co_case_change_0.50', '_co_case_change_0.75', '_co_status_change_0.25', '_co_status_change_0.50', '_co_status_change_0.75']
    #
    # corpus_list2 = ['case_status_w2_0.50_0.25_200', 'case_status_w2_0.50_0.25_100', 'case_status_w2_0.50_0.25_50', 'case_status_w2_0.50_0.25_10']
    #
    # corpus_list = ['all']
    # model_list = ['glove']
    #
    # for model_name in model_list:
    #     model = LoadModel(model_name)
    #
    #     for mode in mode_list:
    #         for corpus in corpus_list:
    #             for corpus2 in corpus_list2:
    #                 try:
    #                     chekck(mode, corpus, model, corpus2)
    #                 except:
    #                     pass



if __name__ == '__main__':
    main()