import pickle
import logging
import sys, os
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

from ptm.rtm import RelationalTopicModel
from ptm.utils import convert_cnt_to_list, get_top_words
from allcoherence import *
from evaluation import LoadModel, calc_cohe


def get_data(corpus, aft):
    path_name = './corpus_data/'

    doc_ids = []
    text_name = path_name + 'doc_ids_{}.txt'.format(corpus)
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split()
            line = list(map(int, line))
            doc_ids.append(line)

    doc_cnt = []
    text_name = path_name + 'doc_cnt_{}.txt'.format(corpus)
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split()
            line = list(map(int, line))
            doc_cnt.append(line)

    doc_links = []
    text_name = path_name + 'link_data_{}{}.txt'.format(corpus, aft)
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split()
            line = list(map(int, line))
            doc_links.append(line)

    index2word = {}
    voca = []
    text_name = path_name + 'index2word_{}.txt'.format(corpus)
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split('\t')
            if len(line) == 1:
                voca.append('')
                continue
            ind, word = line[0], line[1]
            voca.append(word)
            index2word.update({ind : word})

    return doc_ids, doc_cnt, doc_links, voca, index2word

from collections import defaultdict

def get_weight(doc_num, journal_mode):
    weighted_data = defaultdict(list)
    if not journal_mode:return [0 for i in range(doc_num)]
    text_name = 'journal_data/notice_ref.tsv'
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split('\t')
            node = line[1]
            weighted_data[node] = []


    text_name = 'journal_data/weight_data.txt'
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split('\t')
            node1, node2, score1, score2, score3 = line
            score1, score2, score3 = float(score1), float(score2), float(score3)
            if journal_mode == 1:
                pick_up_score = score1
            elif journal_mode == 2:
                pick_up_score = score1
            elif journal_mode == 3:
                pick_up_score = score1 + score2
            elif journal_mode == 4:
                pick_up_score = score1 * score2
            elif journal_mode == 5:
                pick_up_score = score1 * score2/(score1 + score2 -score2 * score1 )
            elif journal_mode == 6:
                pick_up_score = max(0,score1 + score2-1)
            elif journal_mode == 7:
                pick_up_score = max(score1 , score2)
            elif journal_mode == 8:
                pick_up_score = min(score1 , score2)
            elif journal_mode == 9:
                pick_up_score = score1*score2/(1+(1-score1)*(1- score2))
            weighted_data[node1].append(pick_up_score)
            weighted_data[node2].append(pick_up_score)
    for node in weighted_data.keys():
        if weighted_data[node] == []:
            weighted_data[node] = 0
        else:
            weighted_data[node] = sum(weighted_data[node]) / len(weighted_data[node])
    weighted_data = weighted_data.values()
    weighted_data = list(weighted_data) + [0 for i in range(doc_num - len(weighted_data))]
    return weighted_data

from numpy import dot
from numpy.linalg import norm
from gensim.models import Word2Vec
def get_weight_node2vec(doc_num, journal_mode):
    weighted_data = defaultdict(list)
    if not journal_mode:return [0 for i in range(doc_num)]
    text_name = 'journal_data/notice_ref.tsv'
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split('\t')
            node = line[1]
            weighted_data[node] = []
    node2vec_p=Word2Vec.load("models/node2vec_case.model")
    node2vec_s=Word2Vec.load("models/node2vec_status.model")
    text_name = 'journal_data/weight_data.txt'
    with open(text_name, 'r') as f:
        for line in f:
            line = line.strip().split('\t')
            node1, node2, score1, score2, score3 = line
            try:
                score1=float(dot(node2vec_p[node1], node2vec_p[node2])/(norm(node2vec_p[node1])*norm(node2vec_p[node2])))
                score2=float(dot(node2vec_s[node1], node2vec_s[node2])/(norm(node2vec_s[node1])*norm(node2vec_s[node2])))
            except:continue
           
            if journal_mode == 1:
                pick_up_score = score1
            elif journal_mode == 2:
                pick_up_score = score1
            elif journal_mode == 3:
                pick_up_score = score1 + score2
            elif journal_mode == 4:
                pick_up_score = score1 * score2
            elif journal_mode == 5:
                pick_up_score = score1 * score2/(score1 + score2 -score2 * score1 )
            elif journal_mode == 6:
                pick_up_score = max(0,score1 + score2-1)
            elif journal_mode == 7:
                pick_up_score = max(score1 , score2)
            elif journal_mode == 8:
                pick_up_score = min(score1 , score2)
            elif journal_mode == 9:
                pick_up_score = score1*score2/(1+(1-score1)*(1- score2))
            weighted_data[node1].append(pick_up_score)
            weighted_data[node2].append(pick_up_score)
    for node in weighted_data.keys():
        if weighted_data[node] == []:
            weighted_data[node] = 0
        else:
            weighted_data[node] = sum(weighted_data[node]) / len(weighted_data[node])
    weighted_data = weighted_data.values()
    weighted_data = list(weighted_data) + [0 for i in range(doc_num - len(weighted_data))]
    return weighted_data
def make_dir(corpus):
    try:
        os.mkdir('./{}/'.format(corpus))
    except:
        print('folder is already existing.')

def main():
    logger = logging.getLogger('RelationalTopicModel')
    # logger.propagate=False

    # %matplotlib inline

    # doc_ids = pickle.load(open('./python-topic-model/data/cora/doc_ids.pkl', 'rb'))
    # doc_cnt = pickle.load(open('./python-topic-model//data/cora/doc_cnt.pkl', 'rb'))
    # doc_links = pickle.load(open('./python-topic-model//data/cora/doc_links_sym.pkl', 'rb'))
    # voca = pickle.load(open('./python-topic-model//data/cora/voca.pkl', 'rb'))

    journal_mode = 0
    if len(sys.argv)>2:journal_mode=int(sys.argv[2])
    corpus = 'all'
    path_data = 'rtm-n2v-{}'.format(journal_mode)
    max_iter = 10


    make_dir(corpus)
    make_dir('output/{}'.format(path_data))
    aft = '_co'

    doc_ids, doc_cnt, doc_links, voca, index2word = get_data(corpus, aft)
    data_name = corpus

    embedding_name = '/mnt/nfs/nas4/cicoda_tmp/cicoda_tmp/Osaka/data/glove.840B.300d.txt'
    
    logger.info('Loading glove model...')
    # glove = LoadModel(embedding_name)
    if len(sys.argv)>1:num_lis=[int(sys.argv[1])]
    else:num_lis=[10, 50, 100,200]
    for n_topic in num_lis:
        # aaa = 10
        n_doc = len(doc_ids)
        # n_topic = aaa
        n_voca = len(voca)
        value_lim = '5'

        # print('doc_isd len is', len(doc_ids))
        # print('doc_cnt len is', len(doc_cnt))
        # print('doc_links len is', len(doc_links))
        # print('voca len is', len(voca))

        weighted_data = get_weight_node2vec(len(doc_ids), journal_mode)
        # get_weight(len(doc_ids), journal_mode)
        # print(len(weighted_data))

        # coh=[]
        # for i in range(10):
        model = RelationalTopicModel(n_topic, n_doc, n_voca, verbose=True)
        model.fit(doc_ids, doc_cnt, doc_links, weighted_data, journal_mode, max_iter=max_iter)
        # model.save_model(data_name)
        out_name = '{}.txt'.format(data_name)
        path = './output/{}/{}{}'.format(path_data, str(n_topic), out_name)
        # path = './output/' + path_data + ws + str(n_topic) + out_name
        with open(path, 'w') as out:
            for k in range(n_topic):
                top_words = get_top_words(model.beta, voca, k, 100)
                # print('Topic', k, ':', ','.join(top_words))
                out.write('{}\t{}\n'.format(k, '\t'.join(top_words)))
        # sys.exit()
        # coh=calc_cohe(path, glove)
        # print("coherence:{}".format(coh))
        # coh+=[calc_cohe(path, glove)]
        # print('Avg:{}, std:{}'.format(sum(coh),np.std(coh)))
        docs=[[index2word[str(i)]for i in doc]for doc in doc_ids]
        print('docs ok')
        dictionary = Dictionary(docs)
        corpus = [dictionary.doc2bow(doc) for doc in docs]
        print('corpus ok')
        for metric in ['c_v','u_mass','c_uci']:getScore(path,docs,corpus,dictionary,metric=metric)


if __name__ == '__main__':
    main()

